#!/usr/bin/env python3
# Copyright 2020-present NAVER Corp. Under BSD 3-clause license

import unittest
import os
import os.path as path
import sys
import tempfile
# kapture
import path_to_kapture  # enables import kapture  # noqa: F401
import kapture
import kapture.io.records
from kapture.io.binary import transfer_files_from_dir_link, transfer_files_from_dir_copy
from kapture.utils.paths import path_secure, populate_files_in_dirpath


def make_fake_filenames(root_path: str, post_fix=''):
    filenames = [
        path_secure(path.join(dir1, dir2, filename))
        for dir1 in ['a', 'b']
        for dir2 in [f'{i:02d}' for i in range(3)]
        for filename in [f'{i:02d}' for i in range(3)]
    ]
    filepaths = [path_secure(path.join(root_path, filename + post_fix)) for filename in filenames]
    for filepath in filepaths:
        os.makedirs(path.dirname(filepath), exist_ok=True)
        with open(filepath, 'w') as f:
            f.write(filepath)
    return filenames


class TestRecordCopy(unittest.TestCase):
    def setUp(self):
        self._tempdir = tempfile.TemporaryDirectory()
        self._source_dirpath = path.join(self._tempdir.name, 'source')
        self._dest_dirpath = path.join(self._tempdir.name, 'dest')
        self._filenames = make_fake_filenames(self._source_dirpath)

    def tearDown(self):
        self._tempdir.cleanup()

    def test_populate(self):
        filepaths_retrieved = populate_files_in_dirpath(self._source_dirpath)
        self.assertEqual(set(self._filenames),
                         set(filepaths_retrieved))

    def test_copy_generators(self):
        origin_filepaths = (
            path_secure(path.join(self._source_dirpath, filename))
            for filename in self._filenames)
        expected_filepaths = (
            kapture.io.records.get_image_fullpath(self._dest_dirpath, filename)
            for filename in self._filenames)

        transfer_files_from_dir_copy(
            origin_filepaths,
            expected_filepaths
        )

        for expected_filepath in expected_filepaths:
            self.assertTrue(path.isfile(expected_filepath))
        for origin_filepath in origin_filepaths:
            self.assertTrue(path.isfile(origin_filepath))


class TestRecordLinkAbs(unittest.TestCase):
    def setUp(self):
        self._tempdir = tempfile.TemporaryDirectory()
        self._source_dirpath = path.join(self._tempdir.name, 'source')
        self._dest_dirpath = path.join(self._tempdir.name, 'dest')
        self._filenames = make_fake_filenames(self._source_dirpath)

    def tearDown(self):
        self._tempdir.cleanup()

    @unittest.skipIf(sys.platform.startswith("win"), "Do not work on Windows without admin rights")
    def test_link_abs(self):
        source_filepaths = [
            path_secure(path.join(self._source_dirpath, filename))
            for filename in self._filenames]
        destination_filepaths = [
            kapture.io.records.get_image_fullpath(self._dest_dirpath, filename)
            for filename in self._filenames]
        transfer_files_from_dir_link(
            source_filepaths, destination_filepaths, do_relative_link=False
        )

        for destination_filepath, source_filepath in zip(destination_filepaths, source_filepaths):
            self.assertTrue(path.islink(destination_filepath))
            resolved_path = os.readlink(destination_filepath)
            self.assertEqual(source_filepath, resolved_path)

        for source_filepath in source_filepaths:
            self.assertTrue(path.isfile(source_filepath))


class TestRecordLinkRel(unittest.TestCase):
    def setUp(self):
        self._tempdir = tempfile.TemporaryDirectory()
        self._source_dirpath = path.join(self._tempdir.name, 'source')
        self._dest_dirpath = path.join(self._tempdir.name, 'dest')
        self._filenames = make_fake_filenames(self._source_dirpath)

    def tearDown(self):
        self._tempdir.cleanup()

    @unittest.skipIf(sys.platform.startswith("win"), "Do not work on Windows without admin rights")
    def test_link_rel(self):
        source_filepaths = [
            path_secure(path.join(self._source_dirpath, filename))
            for filename in self._filenames]
        destination_filepaths = [
            kapture.io.records.get_image_fullpath(self._dest_dirpath, filename)
            for filename in self._filenames]
        transfer_files_from_dir_link(
            source_filepaths, destination_filepaths, do_relative_link=True
        )

        for destination_filepath, source_filepath in zip(destination_filepaths, source_filepaths):
            self.assertTrue(path.islink(destination_filepath))
            self.assertNotEqual(source_filepath, os.readlink(destination_filepath))
            resolved_path = path.normpath(
                path.join(path.dirname(destination_filepath), os.readlink(destination_filepath)))
            self.assertEqual(source_filepath, resolved_path)

        for source_filepath in source_filepaths:
            self.assertTrue(path.isfile(source_filepath))


class TestPathComputation(unittest.TestCase):
    def setUp(self) -> None:
        self._tempdir = tempfile.TemporaryDirectory()
        self._kapture_path = self._tempdir.name

    def tearDown(self):
        self._tempdir.cleanup()

    def test_get_image_fullpath_empty(self):
        images_path = kapture.io.records.get_image_fullpath("")
        self.assertEqual(kapture.io.records.RECORD_DATA_DIRNAME, images_path)

    def test_get_image_fullpath(self):
        image_name = "my_image.jpg"
        image_path = kapture.io.records.get_image_fullpath(self._kapture_path, image_name)
        self.assertTrue(image_path.startswith(path_secure(self._kapture_path)), "Image path is under the kapture path")
        self.assertTrue(image_path.endswith(image_name), "Image path end with the image name")


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python3
# Copyright 2020-present NAVER Corp. Under BSD 3-clause license

import os.path as path
import tempfile
import unittest
# kapture
import path_to_kapture  # enables import kapture  # noqa: F401
import kapture
from kapture.io.csv import kapture_from_dir
from kapture.algo.compare import equal_kapture
from tools.kapture_import_nvm import import_nvm


class TestImportNVM(unittest.TestCase):

    def setUp(self):
        samples_folder = path.abspath(path.join(path.dirname(__file__),  '../samples/'))
        self.aachen_folder = path.join(samples_folder, 'Aachen-Day-Night')
        self.aachen_models_folder = path.join(self.aachen_folder, '3D-models')
        self.images_folder = path.join(self.aachen_folder, 'images_upright')
        self.filter = path.join(self.aachen_folder, 'filter.txt')

    def test_import_single_model(self):
        nvm_path = path.join(self.aachen_models_folder, 'aachen_cvpr2018_db.nvm')

        with tempfile.TemporaryDirectory() as tmpdirname:
            import_nvm(nvm_path, self.images_folder, tmpdirname,
                       filter_list_path="", ignore_trajectories=False, add_reconstruction=True,
                       force_overwrite_existing=True)

            expected_kdata = kapture_from_dir(path.join(self.aachen_folder, 'kapture/training'))
            imported_aachen_data = kapture_from_dir(tmpdirname)
            self.assertTrue(equal_kapture(imported_aachen_data, expected_kdata))

        # test without trajectories
        with tempfile.TemporaryDirectory() as tmpdirname:
            import_nvm(nvm_path, self.images_folder, tmpdirname,
                       filter_list_path="", ignore_trajectories=True, add_reconstruction=True,
                       force_overwrite_existing=True)

            expected_kdata = kapture_from_dir(path.join(self.aachen_folder, 'kapture/training'))
            expected_kdata._trajectories = None
            imported_aachen_data = kapture_from_dir(tmpdirname)
            self.assertTrue(equal_kapture(imported_aachen_data, expected_kdata))

        # test without points3d
        with tempfile.TemporaryDirectory() as tmpdirname:
            import_nvm(nvm_path, self.images_folder, tmpdirname,
                       filter_list_path="", ignore_trajectories=False, add_reconstruction=False,
                       force_overwrite_existing=True)

            expected_kdata = kapture_from_dir(path.join(self.aachen_folder, 'kapture/training'))
            expected_kdata._points3d = None
            expected_kdata._keypoints = None
            expected_kdata._observations = None
            imported_aachen_data = kapture_from_dir(tmpdirname)
            self.assertTrue(equal_kapture(imported_aachen_data, expected_kdata))

        # test with filter list
        with tempfile.TemporaryDirectory() as tmpdirname:
            import_nvm(nvm_path, self.images_folder, tmpdirname,
                       filter_list_path=self.filter, ignore_trajectories=True, add_reconstruction=False,
                       force_overwrite_existing=True)

            imported_aachen_data = kapture_from_dir(tmpdirname)
            self.assertTrue(len(imported_aachen_data.records_camera.key_pairs()) == 2)
            images_set = {image_path for _, _, image_path in kapture.flatten(imported_aachen_data.records_camera)}
            self.assertTrue('db/1045.jpg' in images_set)
            self.assertFalse('db/4446.jpg' in images_set)
            self.assertTrue('db/1135.jpg' in images_set)
            self.assertIsNone(imported_aachen_data.trajectories)
            self.assertIsNone(imported_aachen_data.points3d)

    def test_import_two_models(self):
        nvm_path = path.join(self.aachen_models_folder, 'aachen_two_parts.nvm')

        with tempfile.TemporaryDirectory() as tmpdirname:
            import_nvm(nvm_path, self.images_folder, tmpdirname,
                       filter_list_path="", ignore_trajectories=False, add_reconstruction=True,
                       force_overwrite_existing=True)

            expected_kdata = kapture_from_dir(path.join(self.aachen_folder, 'kapture/training'))
            imported_aachen_data = kapture_from_dir(tmpdirname)
            self.assertTrue(equal_kapture(imported_aachen_data, expected_kdata))

        # test without trajectories
        with tempfile.TemporaryDirectory() as tmpdirname:
            import_nvm(nvm_path, self.images_folder, tmpdirname,
                       filter_list_path="", ignore_trajectories=True, add_reconstruction=True,
                       force_overwrite_existing=True)

            expected_kdata = kapture_from_dir(path.join(self.aachen_folder, 'kapture/training'))
            expected_kdata._trajectories = None
            imported_aachen_data = kapture_from_dir(tmpdirname)
            self.assertTrue(equal_kapture(imported_aachen_data, expected_kdata))

        # test without points3d
        with tempfile.TemporaryDirectory() as tmpdirname:
            import_nvm(nvm_path, self.images_folder, tmpdirname,
                       filter_list_path="", ignore_trajectories=False, add_reconstruction=False,
                       force_overwrite_existing=True)

            expected_kdata = kapture_from_dir(path.join(self.aachen_folder, 'kapture/training'))
            expected_kdata._points3d = None
            expected_kdata._keypoints = None
            expected_kdata._observations = None
            imported_aachen_data = kapture_from_dir(tmpdirname)
            self.assertTrue(equal_kapture(imported_aachen_data, expected_kdata))

        # test with filter list
        with tempfile.TemporaryDirectory() as tmpdirname:
            import_nvm(nvm_path, self.images_folder, tmpdirname,
                       filter_list_path=self.filter, ignore_trajectories=True, add_reconstruction=False,
                       force_overwrite_existing=True)

            imported_aachen_data = kapture_from_dir(tmpdirname)
            self.assertTrue(len(imported_aachen_data.records_camera.key_pairs()) == 2)
            images_set = {image_path for _, _, image_path in kapture.flatten(imported_aachen_data.records_camera)}
            self.assertTrue('db/1045.jpg' in images_set)
            self.assertFalse('db/4446.jpg' in images_set)
            self.assertTrue('db/1135.jpg' in images_set)
            self.assertIsNone(imported_aachen_data.trajectories)
            self.assertIsNone(imported_aachen_data.points3d)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python3
# Copyright 2020-present NAVER Corp. Under BSD 3-clause license

import unittest
import numpy as np
import quaternion
import copy
import os.path as path
# kapture
import path_to_kapture  # enables import kapture  # noqa: F401
import kapture
import kapture.io.csv as csv
from kapture.algo.compare import is_distance_within_threshold, pose_transform_distance
from kapture.algo.compare import equal_kapture, equal_sensors, equal_rigs, equal_trajectories, equal_records_camera,\
    equal_records_lidar, equal_records_wifi, equal_records_gnss, equal_poses, equal_keypoints_collections


class TestComparePoseTransform(unittest.TestCase):
    """
    Testing kapture/algorithms/compare.py
    """

    def setUp(self):
        self.rotation_a = quaternion.quaternion(-0.572, 0.198, 0.755, -0.252)
        self.rotation_a_negative = quaternion.quaternion(0.572, -0.198, -0.755, 0.252)

        self.translation_a = [144.801, -74.548, -17.746]

        self.pose_a = kapture.PoseTransform(self.rotation_a, self.translation_a)
        self.pose_a_negative = kapture.PoseTransform(self.rotation_a_negative, self.translation_a)

        self.rotation_b = quaternion.quaternion(0.878, 0.090, 0.374, -0.285)
        self.rotation_b_negative = quaternion.quaternion(-0.878, -0.090, -0.374, 0.285)

        self.translation_b = [4.508,  45.032, -37.840]

        self.pose_b = kapture.PoseTransform(self.rotation_b, self.translation_b)
        self.pose_b_negative = kapture.PoseTransform(self.rotation_b_negative, self.translation_b)

        self.pose_ab = kapture.PoseTransform(self.rotation_a, self.translation_b)
        self.pose_ba = kapture.PoseTransform(self.rotation_b, self.translation_a)

        self.pose_none = kapture.PoseTransform(r=None, t=None)
        self.pose_r_none = kapture.PoseTransform(r=None, t=self.translation_a)
        self.pose_t_none = kapture.PoseTransform(r=self.rotation_a, t=None)

    def test_is_distance_within_threshold(self):
        self.assertTrue(is_distance_within_threshold(pose_transform_distance(self.pose_a, self.pose_a)))
        self.assertTrue(is_distance_within_threshold(pose_transform_distance(self.pose_a_negative,
                                                                             self.pose_a_negative)))
        self.assertTrue(is_distance_within_threshold(pose_transform_distance(self.pose_a, self.pose_a_negative)))
        self.assertTrue(is_distance_within_threshold(pose_transform_distance(self.pose_a,
                                                                             self.pose_a.inverse().inverse())))
        self.assertTrue(is_distance_within_threshold(pose_transform_distance(self.pose_b, self.pose_b_negative)))
        self.assertFalse(is_distance_within_threshold(pose_transform_distance(self.pose_a, self.pose_a.inverse())))
        self.assertFalse(is_distance_within_threshold(pose_transform_distance(self.pose_a_negative,
                                                                              self.pose_a_negative.inverse())))
        self.assertFalse(is_distance_within_threshold(pose_transform_distance(self.pose_a, self.pose_b)))
        self.assertFalse(is_distance_within_threshold(pose_transform_distance(self.pose_a, self.pose_ab)))
        self.assertFalse(is_distance_within_threshold(pose_transform_distance(self.pose_a, self.pose_ba)))
        self.assertFalse(is_distance_within_threshold(pose_transform_distance(self.pose_ab, self.pose_ba)))

    def test_equal_poses(self):
        self.assertTrue(equal_poses(self.pose_none, self.pose_none))
        self.assertFalse(equal_poses(self.pose_none, self.pose_r_none))
        self.assertFalse(equal_poses(self.pose_none, self.pose_t_none))
        self.assertFalse(equal_poses(self.pose_t_none, self.pose_r_none))
        self.assertFalse(equal_poses(self.pose_r_none, self.pose_t_none))
        self.assertTrue(equal_poses(self.pose_t_none, self.pose_t_none))
        self.assertTrue(equal_poses(self.pose_r_none, self.pose_r_none))
        self.assertTrue(equal_poses(self.pose_a, self.pose_a))
        self.assertFalse(equal_poses(self.pose_a, self.pose_b))
        self.assertTrue(equal_poses(self.pose_a, self.pose_a_negative))


class TestCompareGnss(unittest.TestCase):
    """
    Testing kapture/algorithms/compare.py
    """

    def setUp(self):
        self.gnss_a = kapture.RecordsGnss()
        self.gnss_b = kapture.RecordsGnss()

        # build 2 gps track using 2 different insertion methods
        # gnss_a[timestamp, gps_id] = values and gnss_b[timestamp] = {gps_id: values}
        for timestamp in range(3):
            gps_snapshots = {}
            for gps_id in ['gps1', 'gps2']:
                values = np.random.uniform(0., 50., size=(5, )).tolist()
                self.gnss_a[timestamp, gps_id] = kapture.RecordGnss(*values)
                gps_snapshots[gps_id] = kapture.RecordGnss(*values)
            self.gnss_b[timestamp] = gps_snapshots

    def test_compare_gnss_equal(self):
        # check they are effectively equals
        self.assertTrue(equal_records_gnss(self.gnss_a, self.gnss_b))

    def test_compare_gnss_modify(self):
        # then modify one and check its not equal anymore
        self.gnss_b[0, 'gps1'].x = 0
        self.assertFalse(equal_records_gnss(self.gnss_a, self.gnss_b))

    def test_compare_gnss_ablation(self):
        # then modify one and check its not equal anymore
        self.gnss_b[0].pop('gps1')
        self.assertFalse(equal_records_gnss(self.gnss_a, self.gnss_b))

    def test_compare_gnss_addition(self):
        # then modify one and check its not equal anymore
        self.gnss_b[0, 'gps3'] = self.gnss_b[0, 'gps1']
        self.assertFalse(equal_records_gnss(self.gnss_a, self.gnss_b))


class TestCompareM1x(unittest.TestCase):

    def setUp(self):
        self._samples_folder = path.abspath(path.join(path.dirname(__file__), '..', 'samples', 'm1x'))
        self._kapture_data = csv.kapture_from_dir(self._samples_folder)

        rotation_a = quaternion.quaternion(-0.572, 0.198, 0.755, -0.252)
        rotation_b = quaternion.quaternion(0.878, 0.090, 0.374, -0.285)
        translation_a = [144.801, -74.548, -17.746]
        translation_b = [144.701, -73.548, -17.746]
        self._pose_a = kapture.PoseTransform(rotation_a, translation_a)
        self._pose_b = kapture.PoseTransform(rotation_b, translation_b)

        self._rigs = kapture.Rigs()
        self._rigs['rig_a', '144323'] = self._pose_a
        self._rigs['rig_b', '144323'] = self._pose_b

        # self._rig_a = kapture.Rig()
        # self._rig_a['144323'] = self._pose_a
        # self._rig_b = kapture.Rig()
        # self._rig_b['144323'] = self._pose_b

    def test_copy_equal(self):
        kapture_data_copy = copy.deepcopy(self._kapture_data)
        self.assertTrue(equal_kapture(kapture_data_copy, self._kapture_data))

    def test_equal_sensors(self):
        kapture_data_a = copy.deepcopy(self._kapture_data)
        kapture_data_b = copy.deepcopy(self._kapture_data)
        self.assertTrue(equal_sensors(kapture_data_a.sensors, kapture_data_b.sensors))

        kapture_data_a.sensors['7497487'] = kapture.Camera(kapture.CameraType.UNKNOWN_CAMERA, [3882, 382])
        self.assertFalse(equal_sensors(kapture_data_a.sensors, kapture_data_b.sensors))

        kapture_data_b.sensors['7497487'] = kapture.Camera(kapture.CameraType.UNKNOWN_CAMERA, [3882, 382])
        self.assertTrue(equal_sensors(kapture_data_a.sensors, kapture_data_b.sensors))

        kapture_data_b.sensors['7497487'] = kapture.Camera(kapture.CameraType.UNKNOWN_CAMERA, [3882, 383])
        self.assertFalse(equal_sensors(kapture_data_a.sensors, kapture_data_b.sensors))

    def test_equal_rigs(self):

        kapture_data_a = copy.deepcopy(self._kapture_data)
        kapture_data_b = copy.deepcopy(self._kapture_data)
        self.assertTrue(equal_rigs(kapture_data_a.rigs, kapture_data_b.rigs))

        kapture_data_a.rigs['497376'] = self._rigs['rig_a']
        self.assertFalse(equal_rigs(kapture_data_a.rigs, kapture_data_b.rigs))

        kapture_data_b.rigs['497376'] = self._rigs['rig_a']
        self.assertTrue(equal_rigs(kapture_data_a.rigs, kapture_data_b.rigs))

        kapture_data_b.rigs['497376'] = self._rigs['rig_b']
        self.assertFalse(equal_rigs(kapture_data_a.rigs, kapture_data_b.rigs))

    def test_equal_trajectories(self):
        kapture_data_a = copy.deepcopy(self._kapture_data)
        kapture_data_b = copy.deepcopy(self._kapture_data)
        self.assertTrue(equal_trajectories(kapture_data_a.trajectories, kapture_data_b.trajectories))

        kapture_data_a.trajectories[(7497487, '497376')] = self._pose_a
        self.assertFalse(equal_trajectories(kapture_data_a.trajectories, kapture_data_b.trajectories))

        kapture_data_b.trajectories[(7497487, '497376')] = self._pose_a
        self.assertTrue(equal_trajectories(kapture_data_a.trajectories, kapture_data_b.trajectories))

        kapture_data_b.trajectories[(7497487, '497376')] = self._pose_b
        self.assertFalse(equal_trajectories(kapture_data_a.trajectories, kapture_data_b.trajectories))

    def test_equal_records_camera(self):
        kapture_data_a = copy.deepcopy(self._kapture_data)
        kapture_data_b = copy.deepcopy(self._kapture_data)
        self.assertTrue(equal_records_camera(kapture_data_a.records_camera, kapture_data_b.records_camera))

        kapture_data_a.records_camera[(7497487, '7497487')] = 'a/a0.jpg'
        self.assertFalse(equal_records_camera(kapture_data_a.records_camera, kapture_data_b.records_camera))

        kapture_data_b.records_camera[(7497487, '7497487')] = 'a/a0.jpg'
        self.assertTrue(equal_records_camera(kapture_data_a.records_camera, kapture_data_b.records_camera))

        kapture_data_b.records_camera[(7497487, '7497487')] = 'b/b0.jpg'
        self.assertFalse(equal_records_camera(kapture_data_a.records_camera, kapture_data_b.records_camera))

    def test_equal_records_lidar(self):
        kapture_data_a = copy.deepcopy(self._kapture_data)
        kapture_data_b = copy.deepcopy(self._kapture_data)
        self.assertTrue(equal_records_lidar(kapture_data_a.records_lidar, kapture_data_b.records_lidar))

        kapture_data_a.records_lidar[(7497487, '7497487')] = 'a/a0.pct'
        self.assertFalse(equal_records_lidar(kapture_data_a.records_lidar, kapture_data_b.records_lidar))

        kapture_data_b.records_lidar[(7497487, '7497487')] = 'a/a0.pct'
        self.assertTrue(equal_records_lidar(kapture_data_a.records_lidar, kapture_data_b.records_lidar))

        kapture_data_b.records_lidar[(7497487, '7497487')] = 'b/b0.pct'
        self.assertFalse(equal_records_lidar(kapture_data_a.records_lidar, kapture_data_b.records_lidar))

    def test_equal_records_wifi(self):
        kapture_data_a = copy.deepcopy(self._kapture_data)
        kapture_data_b = copy.deepcopy(self._kapture_data)
        # check for deep copy
        self.assertTrue(equal_records_wifi(kapture_data_a.records_wifi, kapture_data_b.records_wifi))

        record_wifi_1 = kapture.RecordWifi({
            '68:72:51:80:52:df': kapture.RecordWifiSignal(frequency=2417, rssi=-33.0, ssid='M1X_PicoM2'),
            '68:9c:e2:e1:b0:60': kapture.RecordWifiSignal(frequency=5765, rssi=-49.0, ssid='@HYUNDAI-WiFi'),
        })
        record_wifi_2 = kapture.RecordWifi({
            '68:72:51:80:52:df': kapture.RecordWifiSignal(frequency=2417, rssi=-35.0, ssid='M1X_PicoM2'),
            '68:9c:e2:e1:b0:60': kapture.RecordWifiSignal(frequency=5765, rssi=-47.0, ssid='@HYUNDAI-WiFi'),
        })
        record_wifi_2_str = kapture.RecordWifi({
            '68:72:51:80:52:df': kapture.RecordWifiSignal(frequency='2417', rssi='-35.0', ssid='M1X_PicoM2'),
            '68:9c:e2:e1:b0:60': kapture.RecordWifiSignal(frequency='5765', rssi='-47.0', ssid='@HYUNDAI-WiFi'),
        })
        # check deep copy just did nothing: recreate the B
        kapture_data_b.records_wifi = kapture.RecordsWifi()
        kapture_data_b.records_wifi[1555398770307, 'AC01324954_wifi'] = record_wifi_1
        kapture_data_b.records_wifi[1555398771307, 'AC01324954_wifi'] = record_wifi_2
        self.assertTrue(equal_records_wifi(kapture_data_a.records_wifi, kapture_data_b.records_wifi))
        # check str arguments are casted to the proper types
        kapture_data_b.records_wifi[1555398771307, 'AC01324954_wifi'] = record_wifi_2_str
        self.assertTrue(equal_records_wifi(kapture_data_a.records_wifi, kapture_data_b.records_wifi))

        # check if some are missing, its not equal
        kapture_data_b.records_wifi = kapture.RecordsWifi()
        kapture_data_b.records_wifi[1555398770307, 'AC01324954_wifi'] = record_wifi_1
        self.assertFalse(equal_records_wifi(kapture_data_a.records_wifi, kapture_data_b.records_wifi))

        # just modify a single frequency
        kapture_data_b = copy.deepcopy(self._kapture_data)
        kapture_data_b.records_wifi[1555398770307, 'AC01324954_wifi']['68:72:51:80:52:df'].frequency += 1
        self.assertFalse(equal_records_wifi(kapture_data_a.records_wifi, kapture_data_b.records_wifi))

        # just modify a single rssi
        kapture_data_b = copy.deepcopy(self._kapture_data)
        kapture_data_b.records_wifi[1555398770307, 'AC01324954_wifi']['68:72:51:80:52:df'].rssi += 2.0
        self.assertFalse(equal_records_wifi(kapture_data_a.records_wifi, kapture_data_b.records_wifi))

        # just modify a single bssid
        kapture_data_b = copy.deepcopy(self._kapture_data)
        dropped = kapture_data_b.records_wifi[1555398770307, 'AC01324954_wifi'].pop('68:72:51:80:52:df')
        self.assertFalse(equal_records_wifi(kapture_data_a.records_wifi, kapture_data_b.records_wifi))
        kapture_data_b.records_wifi[1555398770307, 'AC01324954_wifi']['XX:XX:XX:XX:XX:XX'] = dropped
        self.assertFalse(equal_records_wifi(kapture_data_a.records_wifi, kapture_data_b.records_wifi))

    def test_equal_keypoints(self):
        kapture_data_a = copy.deepcopy(self._kapture_data)
        kapture_data_b = copy.deepcopy(self._kapture_data)
        self.assertTrue(equal_keypoints_collections(kapture_data_a.keypoints, kapture_data_b.keypoints))

        kapture_data_a.keypoints['SIFT'].add('a/a0.pct')
        self.assertFalse(equal_keypoints_collections(kapture_data_a.keypoints, kapture_data_b.keypoints))

        kapture_data_b.keypoints['SIFT'].add('a/a0.pct')
        self.assertTrue(equal_keypoints_collections(kapture_data_a.keypoints, kapture_data_b.keypoints))

        kapture_data_b.keypoints['SIFT'].remove('a/a0.pct')
        kapture_data_b.keypoints['SIFT'].add('b/b0.jpg')
        self.assertFalse(equal_keypoints_collections(kapture_data_a.keypoints, kapture_data_b.keypoints))


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python3
# Copyright 2020-present NAVER Corp. Under BSD 3-clause license

import unittest
import numpy as np
import os
import os.path as path
import tempfile
import warnings
# kapture
import path_to_kapture  # enables import kapture  # noqa: F401
import kapture
import kapture.io.csv as csv
from kapture.io.csv import kapture_linesep
import kapture.io.features
import kapture.algo.compare
from kapture.utils.paths import path_secure


########################################################################################################################
# Pose #################################################################################################################
class TestCsvPose(unittest.TestCase):
    def test_pose_write(self):
        pose = kapture.PoseTransform(r=[1.0, 0.0, 0.0, 0.0], t=[0.0, 0.0, 0.0])
        fields = csv.pose_to_list(pose)
        self.assertListEqual(fields, [1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0])
        pose = kapture.PoseTransform(r=[0.5, 0.5, 0.5, 0.5], t=[0., 0., 0.])
        fields = csv.pose_to_list(pose)
        self.assertListEqual(fields, [0.5, 0.5, 0.5, 0.5, 0.0, 0.0, 0.0])
        pose = kapture.PoseTransform(r=[0.5, 0.5, 0.5, 0.5], t=[4., 2., -2.])
        fields = csv.pose_to_list(pose)
        self.assertListEqual(fields, [0.5, 0.5, 0.5, 0.5, 4., 2., -2.])


########################################################################################################################
# File read/write ######################################################################################################
class TestCsvFile(unittest.TestCase):
    def setUp(self):
        self._tempdir = tempfile.TemporaryDirectory()
        self._temp_filepath = path.join(self._tempdir.name, 'timestamps_data.txt')

    def tearDown(self):
        self._tempdir.cleanup()

    def test_last_line_empty(self):
        with open(self._temp_filepath, 'wt') as fw:
            fw.write('')

        with open(self._temp_filepath, 'r') as fr:
            line = csv.get_last_line(fr)
        self.assertEqual(line, '')

    def test_last_line_small_file(self):
        content = kapture_linesep.join([csv.KAPTURE_FORMAT_1,
                                        '# timestamp, device_id, qw, qx, qy, qz, tx, ty, tz',
                                        '0001, cam0,  1.0,  0.0,  0.0,  0.0,  0.0,  0.0,  0.0',
                                        '0002, cam1,  0.5,  0.5,  0.5,  0.5,  4.0,  2.0, -2.0'
                                        ])
        last_line = '1000, cam2,  1.0,  0.0,  0.0,  0.0,  0.0,  0.0,  0.0'
        with open(self._temp_filepath, 'wt') as fw:
            fw.write(content)
            fw.write(kapture_linesep + last_line)

        with open(self._temp_filepath, 'r') as fr:
            line = csv.get_last_line(fr)
        self.assertEqual(line, last_line)

    def test_last_line_big_file(self):
        content = kapture_linesep.join([csv.KAPTURE_FORMAT_1,
                                        '# timestamp, device_id, qw, qx, qy, qz, tx, ty, tz',
                                        '0001, cam0,  1.0,  0.0,  0.0,  0.0,  0.0,  0.0,  0.0'
                                        ])
        some_line = '0002, cam1,  0.5,  0.5,  0.5,  0.5,  4.0,  2.0, -2.0'
        last_line = '1000, cam2,  1.0,  0.0,  0.0,  0.0,  0.0,  0.0,  0.0'
        # Generate a big file
        with open(self._temp_filepath, 'wt') as fw:
            fw.write(content)
            for i in range(0, 100000):
                fw.write(kapture_linesep + some_line)
            fw.write(kapture_linesep + last_line)
        with open(self._temp_filepath, 'r') as fr:
            line = csv.get_last_line(fr)
        self.assertEqual(line, last_line)


########################################################################################################################
# Sensors ##############################################################################################################
class TestCsvSensors(unittest.TestCase):
    def setUp(self):
        self._tempdir = tempfile.TemporaryDirectory()
        self._temp_filepath = path.join(self._tempdir.name, 'sensors.txt')

    def tearDown(self):
        self._tempdir.cleanup()

    def test_sensor_write(self):
        cam0 = kapture.Camera(name='the_name', camera_type='SIMPLE_PINHOLE', camera_params=[640, 480, 100, 320, 240])
        sensor_fields = csv.sensor_to_list(cam0)
        self.assertIsInstance(sensor_fields, list)
        self.assertEqual(len(sensor_fields), 8)
        self.assertEqual(sensor_fields,
                         ['the_name', kapture.SensorType.camera.name,
                          'SIMPLE_PINHOLE', '640', '480', '100', '320', '240'])

    def test_sensor_file_version(self):
        cam0 = kapture.Camera(name='cam0', camera_type='SIMPLE_PINHOLE', camera_params=[640, 480, 100, 320, 240])
        sensors = kapture.Sensors()
        sensors['cam0'] = cam0
        csv.sensors_to_file(self._temp_filepath, sensors)
        version = csv.get_version_from_csv_file(self._temp_filepath)
        current_version = csv.current_format_version()
        self.assertEqual(current_version, version, "Version correctly stored")

    def test_sensors_write(self):
        cam0 = kapture.Camera(name='cam0', camera_type='SIMPLE_PINHOLE', camera_params=[640, 480, 100, 320, 240])
        cam1 = kapture.Camera(name='cam1', camera_type='SIMPLE_PINHOLE', camera_params=[640, 480, 100, 320, 240])
        formatted_expected = kapture_linesep.join([csv.KAPTURE_FORMAT_1,
                                                   '# sensor_id, name, sensor_type, [sensor_params]+',
                                                   'cam0, cam0, camera, SIMPLE_PINHOLE, 640, 480, 100, 320, 240',
                                                   'cam1, cam1, camera, SIMPLE_PINHOLE, 640, 480, 100, 320, 240',
                                                   ''])
        sensors = kapture.Sensors()
        sensors['cam0'] = cam0
        sensors['cam1'] = cam1
        csv.sensors_to_file(self._temp_filepath, sensors)
        with open(self._temp_filepath, 'rt') as f:
            formatted_actual = ''.join(f.readlines())

        self.assertEqual(formatted_actual, formatted_expected)

    def test_sensors_read(self):
        formatted_expected = kapture_linesep.join([
            '# sensor_id, name, sensor_type, [sensor_params]+',
            'cam0, cam0, camera, SIMPLE_PINHOLE, 640, 480, 100, 320, 240',
            'cam1, cam1, camera, SIMPLE_PINHOLE, 640, 480, 100, 320, 240',
            ''])

        with open(self._temp_filepath, 'wt') as f:
            f.write(formatted_expected)

        sensors = csv.sensors_from_file(self._temp_filepath)
        self.assertIsInstance(sensors, kapture.Sensors)
        self.assertEqual(len(sensors), 2)
        self.assertIn('cam0', sensors)
        self.assertIn('cam1', sensors)
        self.assertEqual('cam0', sensors['cam0'].name)
        self.assertEqual('cam1', sensors['cam1'].name)
        self.assertEqual(kapture.SensorType.camera.name, sensors['cam0'].sensor_type)
        self.assertEqual(kapture.SensorType.camera.name, sensors['cam1'].sensor_type)
        self.assertEqual(6, len(sensors['cam1'].sensor_params))
        self.assertListEqual(sensors['cam0'].sensor_params, ['SIMPLE_PINHOLE', '640', '480', '100', '320', '240'])


########################################################################################################################
# Rigs #################################################################################################################
class TestCsvRigs(unittest.TestCase):
    def setUp(self):
        self._tempdir = tempfile.TemporaryDirectory()
        self._temp_filepath = path.join(self._tempdir.name, 'rigs.txt')

    def tearDown(self):
        self._tempdir.cleanup()

    def test_rig_write(self):
        rigs = kapture.Rigs()
        rigs['rig1', 'cam0'] = kapture.PoseTransform()
        rigs['rig1', 'cam1'] = kapture.PoseTransform(r=[0.5, 0.5, 0.5, 0.5])
        content_expected = kapture_linesep.join([csv.KAPTURE_FORMAT_1,
                                                 '# rig_id, sensor_id, qw, qx, qy, qz, tx, ty, tz',
                                                 'rig1, cam0,  1.0,  0.0,  0.0,  0.0,  0.0,  0.0,  0.0',
                                                 'rig1, cam1,  0.5,  0.5,  0.5,  0.5,  0.0,  0.0,  0.0',
                                                 ''])

        csv.rigs_to_file(self._temp_filepath, rigs)
        with open(self._temp_filepath, 'rt') as f:
            content_actual = ''.join(f.readlines())
        self.assertEqual(content_expected, content_actual)


########################################################################################################################
# Trajectories #########################################################################################################
class TestCsvTrajectories(unittest.TestCase):
    def setUp(self):
        self._tempdir = tempfile.TemporaryDirectory()
        self._temp_filepath = path.join(self._tempdir.name, 'trajectories.txt')

    def tearDown(self):
        self._tempdir.cleanup()

    def test_trajectories_write(self):
        pose1 = kapture.PoseTransform(r=[1.0, 0.0, 0.0, 0.0], t=[0.0, 0.0, 0.0])
        pose2 = kapture.PoseTransform(r=[0.5, 0.5, 0.5, 0.5], t=[4., 2., -2.])
        content_expected = [csv.KAPTURE_FORMAT_1 + kapture_linesep,
                            '# timestamp, device_id, qw, qx, qy, qz, tx, ty, tz' + kapture_linesep,
                            '       0, cam0,  1.0,  0.0,  0.0,  0.0,  0.0,  0.0,  0.0' + kapture_linesep,
                            '       0, cam1,  0.5,  0.5,  0.5,  0.5,  4.0,  2.0, -2.0' + kapture_linesep,
                            '     100, cam2,  1.0,  0.0,  0.0,  0.0,  0.0,  0.0,  0.0' + kapture_linesep
                            ]
        trajectories = kapture.Trajectories()
        timestamp1, timestamp2 = 0, 100
        sensor_id1, sensor_id2, sensor_id3 = 'cam0', 'cam1', 'cam2'
        trajectories[(timestamp1, sensor_id1)] = pose1
        trajectories[(timestamp1, sensor_id2)] = pose2
        trajectories[(timestamp2, sensor_id3)] = pose1

        csv.trajectories_to_file(self._temp_filepath, trajectories)
        with open(self._temp_filepath, 'rt') as f:
            content_actual = f.readlines()

        self.assertListEqual(content_actual, content_expected)

    def test_trajectories_read(self):
        content = [
            '# timestamp, device_id, qw, qx, qy, qz, tx, ty, tz',
            '0, cam0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0',
            '    0, cam1, 0.5, 0.5, 0.5, 0.5, 4.0, 2.0, -2.0',
            '     100,  cam2,  1.0,  0.0,  0.0,  0.0,  0.0,  0.0,  0.0'
        ]

        with open(self._temp_filepath, 'wt') as f:
            f.write(kapture_linesep.join(content))
        device_ids = {'cam0', 'cam1', 'cam2'}
        trajectories = csv.trajectories_from_file(self._temp_filepath, device_ids)
        self.assertIsInstance(trajectories, kapture.Trajectories)
        self.assertEqual(2, len(trajectories.keys()))  # timestamps
        self.assertEqual(3, len(trajectories.key_pairs()))  # timestamp x devices
        self.assertIn(0, trajectories)
        self.assertIn(100, trajectories)
        self.assertIn('cam0', trajectories[0])
        self.assertIn('cam1', trajectories[0])

        pose = kapture.PoseTransform(r=[0.5, 0.5, 0.5, 0.5], t=[4., 2., -2.])
        self.assertAlmostEqual(trajectories[(0, 'cam1')].r_raw, pose.r_raw)
        self.assertAlmostEqual(trajectories[(0, 'cam1')].t_raw, pose.t_raw)


########################################################################################################################
# Gnss ##############################################################################################################
class TestCsvGnss(unittest.TestCase):
    def setUp(self):
        self._tempdir = tempfile.TemporaryDirectory()
        self._sensors_filepath = path.join(self._tempdir.name, 'sensors.txt')
        sensors_content = [
            '# sensor_id, name, sensor_type, [sensor_params]+',
            'gps1, gps_01, gnss, EPSG:4326',
            'gps2, gps_02, gnss, EPSG:4326',
        ]
        with open(self._sensors_filepath, 'wt') as f:
            f.write(kapture_linesep.join(sensors_content))

        self._gnss_filepath = path.join(self._tempdir.name, 'records_gnss.txt')
        gnss_content = [
            '# timestamp, device_id, x, y, z, utc, dop',
            '0, gps1, 28.099134, 49.38892, 8.0, 514850398, 3.0',
            '  1, gps1, 29.099134, 50.38892, 9.0, 514850399, 2.0',
            '  1, gps2, 29.099134, 50.38892, 9.0, 514850399, 2.0',
            '     2,   gps1, 30.099134, 51.38892, 10.0, 514850400, 1.0'
        ]
        with open(self._gnss_filepath, 'wt') as f:
            f.write(kapture_linesep.join(gnss_content))

    def tearDown(self):
        self._tempdir.cleanup()

    def test_records_gnss_read(self):
        sensors = csv.sensors_from_file(self._sensors_filepath)
        epsg_codes = {sensor_id: sensor.sensor_params[0]
                      for sensor_id, sensor in sensors.items()
                      if sensor.sensor_type == kapture.SensorType.gnss.name}
        self.assertDictEqual({'gps1': 'EPSG:4326', 'gps2': 'EPSG:4326'}, epsg_codes)

        records_gnss = csv.records_gnss_from_file(self._gnss_filepath)
        self.assertEqual(3, len(records_gnss))
        self.assertEqual(4, len(sorted(kapture.flatten(records_gnss))))
        self.assertIn((2, 'gps1'), records_gnss)
        self.assertEqual(51.388920, records_gnss[2, 'gps1'].y)
        self.assertEqual(30.099134, records_gnss[2, 'gps1'].x)
        self.assertEqual(10.000000, records_gnss[2, 'gps1'].z)
        self.assertEqual(514850400, records_gnss[2, 'gps1'].utc)
        self.assertEqual(1.0, records_gnss[2, 'gps1'].dop)

    def test_records_gnss_read_write_read(self):
        sensors = csv.sensors_from_file(self._sensors_filepath)
        records_gnss = csv.records_gnss_from_file(self._gnss_filepath)

        second_sensors_filepath = self._sensors_filepath + '.cpy'
        second_gnss_filepath = self._gnss_filepath + '.cpy'
        csv.sensors_to_file(second_sensors_filepath, sensors)
        csv.records_gnss_to_file(second_gnss_filepath, records_gnss)
        second_sensors = csv.sensors_from_file(second_sensors_filepath)
        second_records_gnss = csv.records_gnss_from_file(second_gnss_filepath)
        self.assertEqual(len(sensors), len(second_sensors))
        # self.assertEqual(sensors['gps1'], second_sensors['gps1'])
        self.assertEqual(len(records_gnss), len(second_records_gnss))


########################################################################################################################
# Records ##############################################################################################################
class TestCsvRecords(unittest.TestCase):
    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_records_read(self):
        pass


########################################################################################################################
# keypoints ############################################################################################################
class TestCsvKeypoints(unittest.TestCase):
    def setUp(self):
        self._tempdir = tempfile.TemporaryDirectory()
        self._kapture_dirpath = self._tempdir.name
        self._keypoints_dirpath = path.join(self._kapture_dirpath, 'reconstruction', 'keypoints')
        self._keypoints_type = 'SIFT'

    def tearDown(self):
        self._tempdir.cleanup()

    def test_keypoints_read_from_files(self):
        images_ids = set(f'cam{cam}/{timestamp:05d}.jpg'
                         for cam in range(2)
                         for timestamp in range(2))
        # make up keypoints files
        keypoints_config_filepath = path.join(self._keypoints_dirpath, self._keypoints_type, 'keypoints.txt')
        os.makedirs(path.dirname(keypoints_config_filepath), exist_ok=True)
        with open(keypoints_config_filepath, 'wt') as f:
            f.write('SIFT, float, 4')
        keypoints_fullpaths = [
            path_secure(path.join(self._keypoints_dirpath, self._keypoints_type, image_id + '.kpt'))
            for image_id in images_ids
        ]
        for keypoints_fullpath in keypoints_fullpaths:
            os.makedirs(path.dirname(keypoints_fullpath), exist_ok=True)
            with open(keypoints_fullpath, 'wt') as f:
                f.write(' ')

        # lock and load
        keypoints = csv.keypoints_from_dir(self._keypoints_type, self._kapture_dirpath, None)

        # check
        self.assertEqual('SIFT', keypoints.type_name)
        self.assertEqual(4, len(keypoints))
        keypoints_filepaths = kapture.io.features.keypoints_to_filepaths(keypoints,
                                                                         self._keypoints_type,
                                                                         self._kapture_dirpath)
        image_filenames_expected = {path_secure(os.path.join(f'cam{ci}', f'{ts:05d}.jpg'))
                                    for ci in [0, 1] for ts in [0, 1]}
        feature_filepaths_expected = {
            path_secure(os.path.join(f'{self._kapture_dirpath}', 'reconstruction',
                                     'keypoints', self._keypoints_type, f'cam{ci}', f'{ts:05d}.jpg.kpt'))
            for ci in [0, 1] for ts in [0, 1]}

        self.assertEqual(image_filenames_expected, set(keypoints_filepaths.keys()))
        self.assertEqual(feature_filepaths_expected, set(keypoints_filepaths.values()))

    def test_keypoints_read_from_images(self):
        # Create
        images_ids = set(f'cam{cam}/{timestamp:05d}.jpg'
                         for cam in range(2)
                         for timestamp in range(2))
        keypoints_config_filepath = path.join(self._keypoints_dirpath, self._keypoints_type, 'keypoints.txt')
        os.makedirs(path.dirname(keypoints_config_filepath), exist_ok=True)
        with open(keypoints_config_filepath, 'wt') as f:
            f.write('SIFT, float, 4')

        # lock and load
        keypoints = csv.keypoints_from_dir(self._keypoints_type, self._kapture_dirpath, images_ids)

        # check its empty
        self.assertEqual('SIFT', keypoints.type_name)
        self.assertEqual(0, len(keypoints))

        valid = kapture.io.features.keypoints_check_dir(keypoints, self._keypoints_type, self._kapture_dirpath)
        self.assertTrue(valid)

        # create actual files
        for images_id in images_ids:
            keypoint_filepath = path.join(self._keypoints_dirpath, self._keypoints_type, images_id + '.kpt')
            os.makedirs(path.dirname(keypoint_filepath), exist_ok=True)
            with open(keypoint_filepath, 'wt') as f:
                f.write('')

        # lock and load again
        keypoints = csv.keypoints_from_dir(self._keypoints_type, self._kapture_dirpath, images_ids)
        self.assertEqual('SIFT', keypoints.type_name)
        self.assertEqual(4, len(keypoints))

        keypoints_filepaths = kapture.io.features.keypoints_to_filepaths(keypoints,
                                                                         self._keypoints_type,
                                                                         self._kapture_dirpath)
        image_filenames_expected = {f'cam{ci}/{ts:05d}.jpg'
                                    for ci in [0, 1] for ts in [0, 1]}
        feature_filepaths_expected = {
            path_secure(f'{self._kapture_dirpath}/reconstruction/keypoints'
                        f'/{self._keypoints_type}/cam{ci}/{ts:05d}.jpg.kpt')
            for ci in [0, 1] for ts in [0, 1]}

        self.assertEqual(image_filenames_expected, set(keypoints_filepaths))
        self.assertEqual(feature_filepaths_expected, set(keypoints_filepaths.values()))

        valid = kapture.io.features.keypoints_check_dir(keypoints, self._keypoints_type, self._kapture_dirpath)
        self.assertTrue(valid)

        # destroy files and check
        os.remove(path.join(self._keypoints_dirpath, self._keypoints_type, 'cam0/00000.jpg.kpt'))
        valid = kapture.io.features.keypoints_check_dir(keypoints, self._keypoints_type, self._kapture_dirpath)
        self.assertFalse(valid)


########################################################################################################################
# points3d #############################################################################################################
class TestCsvPoints3d(unittest.TestCase):
    def setUp(self):
        self._tempdir = tempfile.TemporaryDirectory()

    def tearDown(self):
        self._tempdir.cleanup()

    def test_points_to_files(self):
        data = np.array([
            [0, 1, 0, 0, 0, 0],
            [1, 0, 0, 255, 0, 0],
            [0, 0, 1, 255, 0, 255],
        ])
        points3d = kapture.Points3d(data)

        filepath = path.join(self._tempdir.name, 'points3d.txt')
        csv.points3d_to_file(filepath, points3d)
        with open(filepath, 'rt') as file:
            lines = file.readlines()

        self.assertEqual(5, len(lines))
        self.assertTrue(lines[0].startswith('#'))
        self.assertTrue(lines[1].startswith('#'))
        first_line = [float(f) for f in lines[2].split(',')]
        self.assertEqual(6, len(first_line))
        self.assertAlmostEqual([0.0, 1.0, 0.0, 0.0, 0.0, 0.0], first_line)

    def test_points_from_empty_file(self):
        filepath = path.join(self._tempdir.name, 'points3d.txt')
        with open(filepath, 'wt') as file:
            file.write(csv.KAPTURE_FORMAT_1 + kapture_linesep)
            file.write('# X, Y, Z, R, G, B' + kapture_linesep)
        # prevent numpy showing warning
        with warnings.catch_warnings():
            warnings.simplefilter("ignore")
            points3d = csv.points3d_from_file(filepath)
        self.assertEqual(0, points3d.shape[0])

    def test_points_from_single_line_file(self):
        filepath = path.join(self._tempdir.name, 'points3d.txt')
        with open(filepath, 'wt') as file:
            file.write(csv.KAPTURE_FORMAT_1 + kapture_linesep)
            file.write('# X, Y, Z, R, G, B' + kapture_linesep)
            file.write('0, 1.0, 0.0, 255, .0, 150' + kapture_linesep)
        points3d = csv.points3d_from_file(filepath)
        self.assertEqual(1, points3d.shape[0])
        self.assertAlmostEqual([0.0, 1.0, 0.0, 255.0, .0, 150.0], points3d[0])

    def test_points_from_file(self):
        filepath = path.join(self._tempdir.name, 'points3d.txt')
        with open(filepath, 'wt') as file:
            file.write(csv.KAPTURE_FORMAT_1 + kapture_linesep)
            file.write('# X, Y, Z, R, G, B' + kapture_linesep)
            file.write('0, 1.0, 0.0, 255, .0, 150' + kapture_linesep)
            file.write('-10, 0, 0.0, 25, .0, 150' + kapture_linesep)
            file.write('1545, 0, 0.0, 25, .0, 150' + kapture_linesep)
        points3d = csv.points3d_from_file(filepath)
        self.assertEqual(3, points3d.shape[0])
        self.assertAlmostEqual([0.0, 1.0, 0.0, 255.0, .0, 150.0], points3d[0])
        self.assertAlmostEqual([1545.0, 0.0, 0.0, 25.0, .0, 150.0], points3d[2])


########################################################################################################################
# Observations #########################################################################################################
class TestCsvObservations(unittest.TestCase):
    def setUp(self):
        self._tempdir = tempfile.TemporaryDirectory()
        self._observations_expected_filepath = path.join(self._tempdir.name, 'expected', 'observations.txt')
        self._observations_actual_filepath = path.join(self._tempdir.name, 'actual', 'observations.txt')
        # creates ground truth couple data/file
        self._observations_expected = kapture.Observations({
            0: {'SIFT': [('image1.jpg', 0), ('image2.jpg', 0)]},
            2: {'SIFT': [('image1.jpg', 2), ('image2.jpg', 3)]}
        })
        self._observations_csv_expected = kapture_linesep.join([csv.KAPTURE_FORMAT_1,
                                                                "# point3d_id, keypoints_type,"
                                                                " [image_path, feature_id]*",
                                                                "0, SIFT, image1.jpg, 0, image2.jpg, 0",
                                                                "2, SIFT, image1.jpg, 2, image2.jpg, 3"])
        self._observations_csv_expected += kapture_linesep
        os.makedirs(path.dirname(self._observations_expected_filepath), exist_ok=True)
        with open(self._observations_expected_filepath, 'wt') as file:
            file.write(self._observations_csv_expected)

    def tearDown(self):
        self._tempdir.cleanup()

    def test_observations_to_file(self):
        csv.observations_to_file(self._observations_actual_filepath, self._observations_expected)
        with open(self._observations_actual_filepath, 'rt') as file:
            lines = file.readlines()
            content_actual = ''.join(lines)
        self.assertEqual(self._observations_csv_expected, content_actual)

    def test_observations_from_file(self):
        observations_actual = csv.observations_from_file(self._observations_expected_filepath)
        self.assertEqual(self._observations_expected, observations_actual)

    def test_observations_from_file_limited(self):
        observations_actual = csv.observations_from_file(self._observations_expected_filepath, {'SIFT': {'image1.jpg'}})
        self.assertEqual(2, len(observations_actual))
        self.assertNotEqual(self._observations_expected, observations_actual)


########################################################################################################################
# Kapture ##############################################################################################################
class TestCsvKapture(unittest.TestCase):
    def setUp(self):
        self._tempdir = tempfile.TemporaryDirectory()

    def tearDown(self):
        self._tempdir.cleanup()

    def test_kapture_write(self):
        kapture_data = kapture.Kapture()

        # test it is not writing files for undefined parts
        csv.kapture_to_dir(self._tempdir.name, kapture_data)
        self.assertFalse(path.exists(path.join(self._tempdir.name, 'sensors', 'sensors.txt')))
        self.assertFalse(path.exists(path.join(self._tempdir.name, 'sensors', 'trajectories.txt')))
        self.assertFalse(path.exists(path.join(self._tempdir.name, 'sensors', 'rigs.txt')))

        # test it is actually writing files for parts
        kapture_data.sensors = kapture.Sensors()
        kapture_data.trajectories = kapture.Trajectories()
        kapture_data.rigs = kapture.Rigs()
        csv.kapture_to_dir(self._tempdir.name, kapture_data)
        self.assertTrue(path.exists(path.join(self._tempdir.name, 'sensors', 'sensors.txt')))
        self.assertTrue(path.exists(path.join(self._tempdir.name, 'sensors', 'trajectories.txt')))
        self.assertTrue(path.exists(path.join(self._tempdir.name, 'sensors', 'rigs.txt')))
        # TODO: using samples

    def test_kapture_write_read(self):
        kapture_data_expected = kapture.Kapture()
        kapture_data_expected.sensors = kapture.Sensors()
        kapture_data_expected.trajectories = kapture.Trajectories()
        kapture_data_expected.rigs = kapture.Rigs()
        csv.kapture_to_dir(self._tempdir.name, kapture_data_expected)
        # TODO: using samples

    def test_kapture_format_version_from_disk(self):
        kapture_data = kapture.Kapture()
        kapture_data.sensors = kapture.Sensors()
        csv.kapture_to_dir(self._tempdir.name, kapture_data)
        version = csv.kapture_format_version(self._tempdir.name)
        self.assertEqual(csv.current_format_version(), version, "We have the current version")

    def test_kapture_format_version_memory(self):
        kapture_data = kapture.Kapture()
        kapture_data.sensors = kapture.Sensors()
        csv.kapture_to_dir(self._tempdir.name, kapture_data)
        kapture_read = csv.kapture_from_dir(self._tempdir.name)
        self.assertEqual(csv.current_format_version(), kapture_read.format_version, "We have the current version")


########################################################################################################################
# M1X ##################################################################################################################
class TestCsvM1x(unittest.TestCase):
    def setUp(self):
        self._tempdir = tempfile.TemporaryDirectory()
        self._samples_folder = path.abspath(path.join(path.dirname(__file__), '..', 'samples', 'm1x'))
        self._kapture_data = csv.kapture_from_dir(self._samples_folder)

    def tearDown(self):
        self._tempdir.cleanup()

    def test_sensors_read_file(self):
        self.assertIsInstance(self._kapture_data.sensors, kapture.Sensors)
        self.assertEqual(len(self._kapture_data.sensors), 14)
        self.assertIn('lidar0', self._kapture_data.sensors)
        self.assertIn('22970291', self._kapture_data.sensors)
        self.assertIn('AC01324969', self._kapture_data.sensors)
        self.assertIn('AC01324954_wifi', self._kapture_data.sensors)
        self.assertIn('AC01324954_bluetooth', self._kapture_data.sensors)
        self.assertEqual('horizontal', self._kapture_data.sensors['lidar1'].name)
        self.assertEqual(kapture.SensorType.camera.name, self._kapture_data.sensors['22970291'].sensor_type)
        self.assertEqual(kapture.SensorType.lidar.name, self._kapture_data.sensors['lidar0'].sensor_type)
        self.assertEqual(15, len(self._kapture_data.sensors['22970291'].sensor_params))

    def test_rigs_read_file(self):
        self.assertIsInstance(self._kapture_data.rigs, kapture.Rigs)
        # there is a single rig called "rig"
        self.assertEqual(len(self._kapture_data.rigs.keys()), 1)
        self.assertIn('rig', self._kapture_data.rigs)
        # there is 13 sensors in this rig
        self.assertEqual(len(self._kapture_data.rigs['rig']), 14)
        self.assertIn('lidar0', self._kapture_data.rigs['rig'])

    def test_trajectories_read_file(self):
        self.assertIsInstance(self._kapture_data.trajectories, kapture.Trajectories)
        # there is 9 timestamps
        self.assertEqual(len(self._kapture_data.trajectories.keys()), 9)
        # there is also 9 timestamps+device
        self.assertEqual(len(self._kapture_data.trajectories.key_pairs()), 9)

    def test_records_camera_read_file(self):
        self.assertEqual(8, len(self._kapture_data.records_camera.keys()))
        self.assertEqual(18, len(self._kapture_data.records_camera.key_pairs()))
        self.assertIn(1555399760151000, self._kapture_data.records_camera)
        self.assertIn(1555399761779869, self._kapture_data.records_camera)
        self.assertIn('AC01324954', self._kapture_data.records_camera[1555399760151000])
        self.assertEqual('AC01324954/AC01324954_1555399760151000.jpg',
                         self._kapture_data.records_camera[1555399760151000]['AC01324954'])

    def test_records_wifi_read_file(self):
        self.assertEqual(2, len(self._kapture_data.records_wifi.keys()))
        self.assertEqual(2, len(self._kapture_data.records_wifi.key_pairs()))
        self.assertIn(1555398770307, self._kapture_data.records_wifi)
        self.assertIn('AC01324954_wifi', self._kapture_data.records_wifi[1555398770307])
        record_wifi_expected = kapture.RecordWifi({
            '68:72:51:80:52:df': kapture.RecordWifiSignal(frequency=2417, rssi=-33.0, ssid='M1X_PicoM2'),
            '68:9c:e2:e1:b0:60': kapture.RecordWifiSignal(frequency=5765, rssi=-49, ssid='@HYUNDAI-WiFi')
        })
        # compare representation, to be robust to ?????
        self.assertEqual(str(record_wifi_expected),
                         str(self._kapture_data.records_wifi[1555398770307, 'AC01324954_wifi']))
        record_wifi_expected = kapture.RecordWifi({
            '68:72:51:80:52:df': kapture.RecordWifiSignal(frequency=2417, rssi=-35, ssid='M1X_PicoM2'),
            '68:9c:e2:e1:b0:60': kapture.RecordWifiSignal(frequency=5765, rssi=-47, ssid='@HYUNDAI-WiFi')
        })
        self.assertEqual(str(record_wifi_expected),
                         str(self._kapture_data.records_wifi[1555398771307, 'AC01324954_wifi']))

    def test_records_bluetooth_read_file(self):
        self.assertEqual(2, len(self._kapture_data.records_bluetooth.keys()))
        self.assertEqual(2, len(self._kapture_data.records_bluetooth.key_pairs()))
        self.assertIn(1555398770307, self._kapture_data.records_bluetooth)
        self.assertIn('AC01324954_bluetooth', self._kapture_data.records_bluetooth[1555398770307])
        record_bluetooth_expected = kapture.RecordBluetooth({
            '35:A8:4B:D9:95:06': kapture.RecordBluetoothSignal(rssi=-73.0),
            '6F:80:17:51:5C:16': kapture.RecordBluetoothSignal(rssi=-88, name='MyPhone')
        })
        # compare representation, to be robust to ?????
        self.assertEqual(str(record_bluetooth_expected),
                         str(self._kapture_data.records_bluetooth[1555398770307, 'AC01324954_bluetooth']))
        record_wifi_expected = kapture.RecordWifi({
            '52:BE:1D:75:47:A1': kapture.RecordBluetoothSignal(rssi=-89),
            '94:65:2D:A6:EF:C4': kapture.RecordBluetoothSignal(rssi=-59)
        })
        self.assertEqual(str(record_wifi_expected),
                         str(self._kapture_data.records_bluetooth[1555398771307, 'AC01324954_bluetooth']))

    def test_records_lidar_read_file(self):
        self.assertEqual(3, len(self._kapture_data.records_lidar.keys()))
        self.assertEqual(6, len(self._kapture_data.records_lidar.key_pairs()))
        self.assertEqual(self._kapture_data.records_lidar[1555399760777612, 'lidar0'], 'lidar0/1555399760777612.pcd')
        self.assertEqual(self._kapture_data.records_lidar[1555399760777612, 'lidar1'], 'lidar1/1555399760777612.pcd')
        self.assertEqual(self._kapture_data.records_lidar[1555399760878721, 'lidar0'], 'lidar0/1555399760878721.pcd')
        self.assertEqual(self._kapture_data.records_lidar[1555399760878721, 'lidar1'], 'lidar1/1555399760878721.pcd')
        self.assertEqual(self._kapture_data.records_lidar[1555399760979021, 'lidar0'], 'lidar0/1555399760979021.pcd')
        self.assertEqual(self._kapture_data.records_lidar[1555399760979021, 'lidar1'], 'lidar1/1555399760979021.pcd')

    def test_read_write(self):
        kapture_data = csv.kapture_from_dir(self._samples_folder)
        csv.kapture_to_dir(self._tempdir.name, kapture_data)
        # TODO


########################################################################################################################
# maupertuis ###########################################################################################################
class TestCsvMaupertuis(unittest.TestCase):
    def setUp(self):
        self._tempdir = tempfile.TemporaryDirectory()
        self._samples_folder = path.abspath(path.join(path.dirname(__file__), '..', 'samples', 'maupertuis'))
        self._kapture_dirpath = path.join(self._samples_folder, 'kapture')
        self._features_type = 'SIFT'
        self._kapture_data = csv.kapture_from_dir(self._kapture_dirpath)

    def tearDown(self):
        self._tempdir.cleanup()

    def test_keypoints_from_dir(self):
        keypoints = kapture.io.csv.keypoints_from_dir(self._features_type, self._kapture_dirpath, None)
        self.assertEqual(keypoints.dtype, np.float32)
        self.assertEqual(keypoints.dsize, 6)
        self.assertEqual(set(keypoints), {'03.jpg', '01.jpg', '02.jpg', '00.jpg'})

    def test_descriptors_from_dir(self):
        descriptors = kapture.io.csv.descriptors_from_dir(self._features_type, self._kapture_dirpath, None, None)
        self.assertEqual(descriptors.dtype, np.uint8)
        self.assertEqual(descriptors.dsize, 128)
        self.assertEqual(set(descriptors), {'03.jpg', '01.jpg', '02.jpg', '00.jpg'})

    def test_matches_from_dir(self):
        matches = kapture.io.csv.matches_from_dir(self._features_type, self._kapture_dirpath)
        self.assertEqual({('00.jpg', '03.jpg'), ('01.jpg', '03.jpg'),
                          ('00.jpg', '02.jpg'), ('02.jpg', '03.jpg'),
                          ('01.jpg', '02.jpg'), ('00.jpg', '01.jpg')},
                         set(matches))

    def test_observations_from_file(self):
        image_filenames_with_keypoints = {self._features_type: {'03.jpg', '01.jpg', '02.jpg', '00.jpg'}}
        observations_filepath = kapture.io.csv.get_csv_fullpath(kapture.Observations, self._kapture_dirpath)
        observations = kapture.io.csv.observations_from_file(observations_filepath, image_filenames_with_keypoints)
        self.assertEqual(1039, len(observations))
        # observations[point3d_idx, keypoints_type]= [(image_filename, keypoint_id), (image_filename, keypoint_id), ...]
        observation0 = observations[0, self._features_type]
        self.assertEqual(
            [('01.jpg', 4561), ('02.jpg', 3389), ('00.jpg', 4975), ('03.jpg', 3472)],
            observation0
        )

    def test_overall_read(self):
        # check the overall loading went fine
        self.assertEqual(2, len(self._kapture_data.sensors))
        self.assertEqual(4, len(self._kapture_data.trajectories))
        self.assertEqual(4, len(self._kapture_data.records_camera))
        self.assertEqual(4, len(self._kapture_data.records_gnss))
        self.assertEqual(4, len(self._kapture_data.keypoints[self._features_type]))
        self.assertEqual(4, len(self._kapture_data.descriptors[self._features_type]))
        self.assertEqual(6, len(self._kapture_data.matches[self._features_type]))
        self.assertEqual(1039, len(self._kapture_data.observations))
        self.assertEqual(1039, len(self._kapture_data.points3d))


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python3
# Copyright 2020-present NAVER Corp. Under BSD 3-clause license

import unittest
import os.path as path
import tempfile
# kapture
import path_to_kapture  # enables import kapture  # noqa: F401
from tools.kapture_export_ply import export_ply


class TestPlot(unittest.TestCase):

    def setUp(self):
        samples_folder = path.abspath(path.join(path.dirname(__file__), '../samples/'))
        self.toplot_folder = path.join(samples_folder, 'maupertuis', 'kapture')

    def test_plot(self):
        with tempfile.TemporaryDirectory() as tmpdirname:
            export_ply(
                kapture_path=self.toplot_folder,
                ply_dir_path=tmpdirname,
                axis_length=2.0,
                only=[], skip=[])
            self.assertTrue(path.isfile(path.join(tmpdirname, "points3d.ply")))
            self.assertTrue(path.isfile(path.join(tmpdirname, "trajectories.ply")))


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python3
# Copyright 2020-present NAVER Corp. Under BSD 3-clause license

"""
OpenMVG import and export to kapture unit tests.
"""

import json
import logging
import numpy
import os
import os.path as path
import sys
import tempfile
from typing import Dict, List
import unittest
# kapture
import path_to_kapture  # enables import kapture  # noqa: F401
import kapture
from kapture.algo.compare import equal_poses, equal_kapture
import kapture.io.csv as kcsv
from kapture.io.features import get_descriptors_fullpath, image_descriptors_from_file
from kapture.io.features import get_keypoints_fullpath, image_keypoints_from_file
from kapture.io.records import TransferAction, get_image_fullpath

from kapture.converter.openmvg.import_openmvg import import_openmvg, import_openmvg_sfm_data_json  # noqa: E402
from kapture.converter.openmvg.export_openmvg import export_openmvg  # noqa: E402
from kapture.converter.openmvg.openmvg_commons import OPENMVG_SFM_DATA_VERSION_NUMBER, JSON_KEY
from kapture.converter.openmvg.openmvg_commons import CameraModel

logger = logging.getLogger('openmvg')


class TestOpenMvg(unittest.TestCase):

    # Constants related to the dataset to import
    FIRST_TRAJECTORY_TRANSLATION = [-2.1192500000000005, -0.4327849999999997, -4.07387]
    FIRST_TRAJECTORY_ROTATION = [0.9926160706165561, -0.08600940611886934, 0.0843934060039041, 0.01390940098954065]

    # Constants related to the dataset exported (M1X)
    FIRST_BASLER_SENSOR_ID = "22970285"
    FIRST_BASLER_OPENMVG_ID = 2147483649
    LIDAR1 = "lidar1"

    @classmethod
    def setUpClass(cls) -> None:
        """
        Setup variables for all the tests
        """
        logger.setLevel(logging.CRITICAL)
        cls.isWindows = sys.platform.startswith("win") or sys.platform.startswith("cygwin")
        test_file_path = path.dirname(__file__)
        cls._openmvg_sample_path = path.abspath(path.join(test_file_path, '../samples/t265/Everest_undistorted'))
        cls._kapture_sample_path = path.abspath(path.join(test_file_path, '../samples/m1x'))

    def setUp(self) -> None:
        """
        Setup before every test
        """
        self._tempdir = tempfile.TemporaryDirectory()
        self._kapture_path = path.join(self._tempdir.name, 'from_openmvg')
        os.makedirs(self._kapture_path, exist_ok=True)

    def _verify_data(self, kapture_data) -> None:
        cameras = kapture_data.cameras
        self.assertIsNotNone(cameras, "Cameras exist")
        self.assertEqual(1, len(cameras), "One camera")
        camera = next(iter(cameras.values()))  # just take the first camera defined
        self.assertEqual(camera.camera_type, kapture.CameraType.SIMPLE_RADIAL_FISHEYE, "Type fisheye")
        camera_params = camera.camera_params
        self.assertEqual(848, camera_params[0], "width")
        self.assertEqual(800, camera_params[1], "height")
        records_camera = kapture_data.records_camera
        self.assertEqual(5, len(records_camera), "Number of images")
        first_record = records_camera[0]
        img_path = next(iter(first_record.values()))
        self.assertEqual("images/frame_000000001.jpg", img_path, "Image path")
        trajectories = kapture_data.trajectories
        self.assertEqual(5, len(trajectories), "Trajectories points")
        k_pose6d = next(iter(trajectories[0].values()))  # Kapture.PoseTransform
        ref_pose = kapture.PoseTransform(t=self.FIRST_TRAJECTORY_TRANSLATION, r=self.FIRST_TRAJECTORY_ROTATION)
        self.assertTrue(equal_poses(ref_pose, k_pose6d), "First trajectory pose")
        self.assertIsNone(kapture_data.keypoints, "No keypoints")
        self.assertIsNone(kapture_data.observations, "No observations")
        self.assertIsNone(kapture_data.points3d, "No 3D points")

    def test_openmvg_to_kapture(self) -> None:
        """
        Test the openmvg_to_kapture function on a small openmvg JSON file
        """
        json_file = path.join(self._openmvg_sample_path, 'sfm_data_small.json')
        with open(json_file, 'r') as f:
            sfm_data = json.load(f)
            views_id_to_filenames: Dict[int, str] = {}
            kapture_data = import_openmvg_sfm_data_json(sfm_data, self._kapture_path, views_id_to_filenames)
            self._verify_data(kapture_data)

    def test_import_openmvg(self) -> None:
        """
        Test the import_openmvg function on a small JSON file while linking the images
        """
        self.assertTrue(path.isdir(self._openmvg_sample_path))
        self.assertTrue(path.exists(self._kapture_path), "Kapture directory exists")
        sfm_file = path.join(self._openmvg_sample_path, 'sfm_data_small.json')
        # on windows, without admin rights, fails with OSError: symbolic link privilege not held
        # see https://docs.python.org/3.6/library/os.html#os.symlink
        logger.info(f'Running on "{sys.platform}" which is {"" if self.isWindows else "not"} a Windows platform')
        file_operation = TransferAction.skip if self.isWindows else TransferAction.link_relative
        import_openmvg(sfm_file, None, None, self._kapture_path, file_operation, True)
        #  test presence or absence of kapture files
        cameras_file_path = path.join(self._kapture_path, kcsv.CSV_FILENAMES[kapture.Sensors])
        self.assertTrue(path.isfile(cameras_file_path), "Camera file written")
        rigs_file_path = path.join(self._kapture_path, kcsv.CSV_FILENAMES[kapture.Rigs])
        self.assertFalse(path.isfile(rigs_file_path), "Rigs file should be missing")
        records_file_path = path.join(self._kapture_path, kcsv.CSV_FILENAMES[kapture.RecordsCamera])
        self.assertTrue(path.isfile(records_file_path), "Camera Records file written")
        lidars_file_path = path.join(self._kapture_path, kcsv.CSV_FILENAMES[kapture.RecordsLidar])
        self.assertFalse(path.isfile(lidars_file_path), "Lidar Records file should be missing")
        trajectories_file_path = path.join(self._kapture_path, kcsv.CSV_FILENAMES[kapture.Trajectories])
        self.assertTrue(path.isfile(trajectories_file_path), "Trajectories file written")
        # Reload data and verify
        kapture_data = kcsv.kapture_from_dir(self._kapture_path)
        self._verify_data(kapture_data)
        if not self.isWindows:
            # Test images path
            records_camera_files = kapture_data.records_camera.data_list()
            for name in records_camera_files:
                img_path = get_image_fullpath(self._kapture_path, name)
                self.assertTrue(path.islink(img_path), f"image link {img_path}")

    def test_kapture_to_openmvg(self) -> None:
        """
        Test the kapture_to_openmvg export function on a small kapture dataset
        """
        self.assertTrue(path.exists(self._kapture_sample_path), "Kapture directory exists")
        openmvg_json_file = path.join(self._tempdir.name, 'sfm_export.json')
        openmvg_image_root_path = path.join(self._tempdir.name, 'images')
        export_openmvg(kapture_path=self._kapture_sample_path,
                       openmvg_sfm_data_file_path=openmvg_json_file,
                       openmvg_image_root_path=openmvg_image_root_path,
                       image_action=TransferAction.copy,
                       force=True)
        self.assertTrue(path.isfile(openmvg_json_file), "Openmvg JSON file created")
        with open(openmvg_json_file, 'r') as f:
            sfm_data = json.load(f)
            self.assertEqual(OPENMVG_SFM_DATA_VERSION_NUMBER,
                             sfm_data.get(JSON_KEY.SFM_DATA_VERSION), "Sfm data version number")
            root_path = sfm_data.get(JSON_KEY.ROOT_PATH)
            self.assertIsNotNone(root_path, "Root path exported")
            self.assertEqual(openmvg_image_root_path, root_path, "Root path correct")
            intrinsics = sfm_data.get(JSON_KEY.INTRINSICS)
            self.assertIsNotNone(intrinsics, "Intrinsics")
            self.assertEqual(9, len(intrinsics), "Cameras")
            camera_ids = set()
            # Search for camera 22970285 and lidar1
            basler = None
            lidar = None
            for intrinsic in intrinsics:
                camera_id = intrinsic.get(JSON_KEY.VALUE, {}).get(JSON_KEY.PTR_WRAPPER, {}).get(JSON_KEY.ID)
                camera_ids.add(camera_id)
                if camera_id == self.FIRST_BASLER_OPENMVG_ID:
                    basler = intrinsic.get(JSON_KEY.VALUE)
                elif camera_id == self.LIDAR1:
                    lidar = intrinsic.get(JSON_KEY.VALUE)
            self.assertEqual(9, len(camera_ids), "All camera identifiers are different")
            self.assertIsNotNone(basler, "First basler camera")
            self.assertEqual(CameraModel.pinhole_brown_t2.name,
                             basler.get(JSON_KEY.POLYMORPHIC_NAME), "Polymorphic name")
            camera_params = basler.get(JSON_KEY.PTR_WRAPPER).get(JSON_KEY.DATA).get(JSON_KEY.VALUE0)
            self.assertEqual(2048, camera_params.get(JSON_KEY.WIDTH), "Camera width")
            self.assertEqual(1536, camera_params.get(JSON_KEY.HEIGHT), "Camera height")
            self.assertAlmostEqual(1725.842032333, camera_params.get(JSON_KEY.FOCAL_LENGTH), msg="Focal length")
            self.assertEqual(1024, camera_params.get(JSON_KEY.PRINCIPAL_POINT)[0], "Principal point X")
            self.assertEqual(768, camera_params.get(JSON_KEY.PRINCIPAL_POINT)[1], "Principal point Y")
            self.assertIsNotNone(basler.get(JSON_KEY.PTR_WRAPPER).get(JSON_KEY.DATA).get(JSON_KEY.DISTO_T2),
                                 "Disto_t2 defined")
            # Make sure lidar has not been exported to openMVG
            self.assertIsNone(lidar, "Lidar 1")
            # Recorded images: views in openmvg format
            views = sfm_data.get(JSON_KEY.VIEWS)
            self.assertIsNotNone(views, "Views")
            self.assertEqual(18, len(views), "Recorded images")
            image_record = None
            for view in views:
                if view.get(JSON_KEY.KEY) == 3:
                    image_record = view.get(JSON_KEY.VALUE).get(JSON_KEY.PTR_WRAPPER).get(JSON_KEY.DATA)
                    break
            self.assertIsNotNone(image_record, "4th image record")
            local_path = image_record.get(JSON_KEY.LOCAL_PATH)
            self.assertEqual(self.FIRST_BASLER_SENSOR_ID, local_path, "Local path is the camera id")
            self.assertEqual(camera_params.get(JSON_KEY.WIDTH), image_record.get(JSON_KEY.WIDTH),
                             "Image has camera width")
            self.assertEqual(camera_params.get(JSON_KEY.HEIGHT), image_record.get(JSON_KEY.HEIGHT),
                             "Image has camera height")
            filename = image_record.get(JSON_KEY.FILENAME)
            copied_image_path = path.join(root_path, local_path, filename)
            self.assertTrue(path.isfile(copied_image_path), "Image copied")
            self.assertIsNotNone(filename, "Filename is defined")
            pose_id = image_record.get(JSON_KEY.ID_POSE)
            self.assertIsNotNone(pose_id, "Pose id")
            self.assertTrue(image_record.get(JSON_KEY.USE_POSE_ROTATION_PRIOR), "Use pose rotation prior is true")
            view_center = image_record.get(JSON_KEY.CENTER)
            view_rotation = image_record.get(JSON_KEY.ROTATION)
            self.assertIsNotNone(view_center, "Center of image")
            self.assertIsNotNone(view_rotation, "Rotation of image")
            # poses: extrinsics in openmvg format
            extrinsics = sfm_data.get(JSON_KEY.EXTRINSICS)
            self.assertIsNotNone(extrinsics, "Extrinsics")
            self.assertEqual(10, len(extrinsics), "Trajectory points")
            # Search for the pose of the above image
            pose = None
            for extrinsic in extrinsics:
                if extrinsic.get(JSON_KEY.KEY) == pose_id:
                    pose = extrinsic.get(JSON_KEY.VALUE)
                    break
            self.assertIsNotNone(pose, "Pose for image")
            pose_center = pose.get(JSON_KEY.CENTER)
            pose_rotation = pose.get(JSON_KEY.ROTATION)
            self.assertEqual(view_center, pose_center, "Center are equal")
            self.assertEqual(view_rotation, pose_rotation, "Rotations are equal")

    def tearDown(self) -> None:
        """
        Clean up after every test
        """
        self._tempdir.cleanup()


class TestOpenMvgReconstruction(unittest.TestCase):

    UNIQUE_CAMERA_OPENMVG_ID = 2147483649
    FIRST_IMAGE_NAME = "ChateauMaupertuisTest/MaupertuisTest_01.jpg"
    FIRST_TRAJECTORY_TRANSLATION = [-0.02839049268018459, 0.00255775313260552, 0.02473073308745868]
    FIRST_TRAJECTORY_ROTATION = [0.9999935675163499, 0.0004312835445997, 0.0033445836819604, -0.0012217530116397]
    FEATURE_TYPE = "SIFT"
    DESCRIPTOR_TYPE = FEATURE_TYPE+"_Image_describer"
    KEYPOINTS_TYPE = FEATURE_TYPE+"_Regions"

    @classmethod
    def setUpClass(cls) -> None:
        """
        Setup variables for all the tests
        """
        logger.setLevel(logging.CRITICAL)
        cls.isWindows = sys.platform.startswith("win") or sys.platform.startswith("cygwin")
        test_file_path = path.dirname(__file__)
        cls._openmvg_sample_path = path.abspath(path.join(test_file_path,
                                                          '../samples/maupertuis_openMVG/ChateauMaupertuisTest'))
        cls._kapture_sample_path = path.join(test_file_path,
                                             '../samples/maupertuis_openMVG/kapture_ChateauMaupertuisTest')
        cls._kapture_sample_path = path.abspath(cls._kapture_sample_path)

    def setUp(self) -> None:
        """
        Setup before every test
        """
        self._tempdir = tempfile.TemporaryDirectory()
        self._kapture_path = path.join(self._tempdir.name, 'from_openmvg')
        os.makedirs(self._kapture_path, exist_ok=True)

    def _verify_data(self, kapture_data: kapture.Kapture, kapture_path: str) -> None:
        # Cameras
        cameras = kapture_data.cameras
        self.assertIsNotNone(cameras, "Cameras exist")
        self.assertEqual(1, len(cameras), "One camera")
        camera = next(iter(cameras.values()))  # just take the first camera defined
        self.assertEqual(camera.camera_type, kapture.CameraType.RADIAL, "Type Radial")
        camera_params = camera.camera_params
        self.assertEqual(1600, camera_params[0], "width")
        self.assertEqual(1200, camera_params[1], "height")
        # Images
        records_camera = kapture_data.records_camera
        self.assertEqual(4, len(records_camera), "Number of images")
        first_record = records_camera[0]
        img_path = next(iter(first_record.values()))
        self.assertEqual(self.FIRST_IMAGE_NAME, img_path, "Image path")
        # Poses
        trajectories = kapture_data.trajectories
        self.assertEqual(4, len(trajectories), "Trajectories points")
        k_pose6d = next(iter(trajectories[0].values()))  # Kapture.PoseTransform
        ref_pose = kapture.PoseTransform(t=self.FIRST_TRAJECTORY_TRANSLATION, r=self.FIRST_TRAJECTORY_ROTATION)
        self.assertTrue(equal_poses(ref_pose, k_pose6d), "First trajectory pose")
        # Descriptors
        kapture_descriptors = kapture_data.descriptors[self.DESCRIPTOR_TYPE]
        self.assertEqual(4, len(kapture_descriptors), "Descriptors")
        self.assertEqual(self.FEATURE_TYPE, kapture_descriptors.type_name, "Descriptors feature type")
        self.assertEqual(self.KEYPOINTS_TYPE, kapture_descriptors.keypoints_type, "Descriptors keypoints type")
        self.assertIs(numpy.uint8, kapture_descriptors.dtype, "Descriptors dtype")
        self.assertEqual(128, kapture_descriptors.dsize, "Descriptors dsize")
        descriptors_file_path = get_descriptors_fullpath(self.DESCRIPTOR_TYPE,
                                                         self._kapture_path,
                                                         self.FIRST_IMAGE_NAME)
        first_descriptors_data = image_descriptors_from_file(descriptors_file_path,
                                                             kapture_descriptors.dtype,
                                                             kapture_descriptors.dsize)
        self.assertEqual((1850, 128), first_descriptors_data.shape, "Descriptors shape")
        self.assertEqual(numpy.uint8, first_descriptors_data.dtype, "Descriptors dtype")

        # Keypoints
        kapture_keypoints = kapture_data.keypoints[self.KEYPOINTS_TYPE]
        self.assertEqual(4, len(kapture_keypoints), "Keypoints")
        self.assertEqual(self.FEATURE_TYPE, kapture_keypoints.type_name, "Keypoints feature type")
        self.assertIs(float, kapture_keypoints.dtype, "Keypoints dtype")
        self.assertEqual(4, kapture_keypoints.dsize, "Keypoints dsize")
        keypoints_file_path = get_keypoints_fullpath(self.KEYPOINTS_TYPE,
                                                     kapture_path,
                                                     self.FIRST_IMAGE_NAME,
                                                     None)
        first_keypoints_data = image_keypoints_from_file(keypoints_file_path,
                                                         kapture_keypoints.dtype,
                                                         kapture_keypoints.dsize)
        self.assertEqual(first_keypoints_data.shape, (1850, 4), "Keypoints shape")
        self.assertEqual(first_keypoints_data.dtype, numpy.float64, "Keypoints dtype")

        # Observations
        kapture_observations = kapture_data.observations
        self.assertEqual(len(kapture_observations), 365, "Observations")
        self.assertEqual(kapture_observations.observations_number(), 1228, "Number of observations")
        image_name_subdir = path.dirname(self.FIRST_IMAGE_NAME)
        for per_feature_observations in kapture_observations.values():
            for keypoints_type, observations_list in per_feature_observations.items():
                self.assertEqual(keypoints_type, self.KEYPOINTS_TYPE, "Observation keypoints type")
                self.assertLessEqual(len(observations_list), 4, "observations per 3D point")
                for observation in observations_list:
                    self.assertTrue(observation[0].startswith(image_name_subdir), "Observation image name subdirectory")
        # 3D Points
        kapture_points3d = kapture_data.points3d
        self.assertEqual(kapture_points3d.shape, (372, kapture.Points3d.XYZ_ONLY), "3D points shape")

    def test_import_openmvg_reconstruction(self) -> None:
        """
        Test the import_openmvg function on a reconstruction JSON file with keypoints, 3D points and observations
        """
        self.assertTrue(path.isdir(self._openmvg_sample_path))
        self.assertTrue(path.exists(self._kapture_path), "Kapture directory exists")
        sfm_file = path.join(self._openmvg_sample_path, 'reconstruction_global', 'sfm_data.json')
        openmvg_matches_dir = path.join(self._openmvg_sample_path, 'matches')
        openmvg_matches_file = path.join(openmvg_matches_dir, 'matches.f.txt')
        import_openmvg(sfm_file, openmvg_matches_dir, openmvg_matches_file, self._kapture_path, TransferAction.skip)
        #  test presence or absence of kapture files
        cameras_file_path = kcsv.get_csv_fullpath(kapture.Sensors, self._kapture_path)
        self.assertTrue(path.isfile(cameras_file_path), "Camera file written")
        rigs_file_path = kcsv.get_csv_fullpath(kapture.Rigs, self._kapture_path)
        self.assertFalse(path.isfile(rigs_file_path), "Rigs file should be missing")
        records_file_path = kcsv.get_csv_fullpath(kapture.RecordsCamera, self._kapture_path)
        self.assertTrue(path.isfile(records_file_path), "Camera Records file written")
        lidars_file_path = kcsv.get_csv_fullpath(kapture.RecordsLidar, self._kapture_path)
        self.assertFalse(path.isfile(lidars_file_path), "Lidar Records file should be missing")
        trajectories_file_path = kcsv.get_csv_fullpath(kapture.Trajectories, self._kapture_path)
        self.assertTrue(path.isfile(trajectories_file_path), "Trajectories file written")
        keypoints_file_path = kcsv.get_feature_csv_fullpath(kapture.Keypoints, self.KEYPOINTS_TYPE, self._kapture_path)
        self.assertTrue(path.isfile(keypoints_file_path), "keypoints file written")
        descriptors_file_path = kcsv.get_feature_csv_fullpath(kapture.Descriptors,
                                                              "SIFT_Image_describer",
                                                              self._kapture_path)
        self.assertTrue(path.isfile(descriptors_file_path), "descriptors file written")
        points3d_file_path = kcsv.get_csv_fullpath(kapture.Points3d, self._kapture_path)
        self.assertTrue(path.isfile(points3d_file_path), "3D points file written")
        observations_file_path = kcsv.get_csv_fullpath(kapture.Observations, self._kapture_path)
        self.assertTrue(path.isfile(observations_file_path), "observations file written")

        # Reload data and verify
        kapture_data = kcsv.kapture_from_dir(self._kapture_path)
        kapture_sample_data = kcsv.kapture_from_dir(self._kapture_sample_path)
        self.assertTrue(equal_kapture(kapture_sample_data, kapture_data), "Created kapture is equal to sample")
        # Compare with kapture sample
        self._verify_data(kapture_data, self._kapture_path)

    def test_kapture_to_openmvg_reconstruction(self) -> None:
        """
        Test the kapture_to_openmvg export function on a small kapture dataset with 3D reconstruction data
        """
        self.assertTrue(path.exists(self._kapture_sample_path), "Kapture directory exists")
        kapture_sample_data = kcsv.kapture_from_dir(self._kapture_sample_path)
        openmvg_json_file = path.join(self._tempdir.name, 'sfm_export_3d.json')
        openmvg_image_root_path = path.join(self._tempdir.name, 'images')
        regions_dir_path = path.join(self._tempdir.name, 'regions')
        export_openmvg(kapture_path=self._kapture_sample_path,
                       openmvg_sfm_data_file_path=openmvg_json_file,
                       openmvg_image_root_path=openmvg_image_root_path,
                       openmvg_regions_dir_path=regions_dir_path,
                       image_action=TransferAction.copy,
                       force=True)
        self.assertTrue(path.isfile(openmvg_json_file), "Openmvg JSON file created")
        with open(openmvg_json_file, 'r') as f:
            sfm_data = json.load(f)
            self.assertEqual(OPENMVG_SFM_DATA_VERSION_NUMBER,
                             sfm_data.get(JSON_KEY.SFM_DATA_VERSION), "Sfm data version number")
            root_path: str = sfm_data.get(JSON_KEY.ROOT_PATH)
            self.assertIsNotNone(root_path, "Root path exported")
            self.assertTrue(root_path.startswith(openmvg_image_root_path), "Root path correct")
            kapture_camera: kapture.Camera = kapture_sample_data.cameras.get('0')
            kapture_camera_params = kapture_camera.camera_params
            intrinsics = sfm_data.get(JSON_KEY.INTRINSICS)
            self.assertIsNotNone(intrinsics, "Intrinsics")
            self.assertEqual(1, len(intrinsics), "Camera")
            intrinsic = intrinsics[0]
            unique_camera = intrinsic.get(JSON_KEY.VALUE)
            self.assertIsNotNone(unique_camera, "Unique camera")
            self.assertEqual(CameraModel.pinhole_radial_k3.name,
                             unique_camera.get(JSON_KEY.POLYMORPHIC_NAME), "Polymorphic name")
            camera_id = unique_camera.get(JSON_KEY.PTR_WRAPPER, {}).get(JSON_KEY.ID)
            self.assertEqual(self.UNIQUE_CAMERA_OPENMVG_ID, camera_id, "Unique openmvg camera id")
            camera_params = unique_camera.get(JSON_KEY.PTR_WRAPPER, {}).get(JSON_KEY.DATA)
            self.assertIsNotNone(camera_params, "Camera params")
            self.assertEqual(kapture_camera_params[0], camera_params.get(JSON_KEY.WIDTH), "Camera width")
            self.assertEqual(kapture_camera_params[1], camera_params.get(JSON_KEY.HEIGHT), "Camera height")
            self.assertAlmostEqual(kapture_camera_params[2], camera_params.get(JSON_KEY.FOCAL_LENGTH), msg="Focal len")
            principal_point = camera_params.get(JSON_KEY.PRINCIPAL_POINT)
            self.assertEqual(kapture_camera_params[3], principal_point[0], "Principal point X")
            self.assertEqual(kapture_camera_params[4], principal_point[1], "Principal point Y")
            disto_k3 = camera_params.get(JSON_KEY.DISTO_K3)
            self.assertIsNotNone(disto_k3, "Disto_k3 defined")
            self.assertEqual(kapture_camera_params[5], disto_k3[0], "Disto K1")
            self.assertEqual(kapture_camera_params[6], disto_k3[1], "Disto K2")
            # Recorded images: views in openmvg format
            views = sfm_data.get(JSON_KEY.VIEWS)
            self.assertIsNotNone(views, "Views")
            self.assertEqual(4, len(views), "Recorded images")
            image_record = None
            for view in views:
                if view.get(JSON_KEY.KEY) == 0:
                    image_record = view.get(JSON_KEY.VALUE).get(JSON_KEY.PTR_WRAPPER).get(JSON_KEY.DATA)
                    break
            self.assertIsNotNone(image_record, "1th image record")
            local_path = image_record.get(JSON_KEY.LOCAL_PATH)
            self.assertEqual("", local_path, "Local path is empty")
            self.assertEqual(camera_params.get(JSON_KEY.WIDTH), image_record.get(JSON_KEY.WIDTH),
                             "Image has camera width")
            self.assertEqual(camera_params.get(JSON_KEY.HEIGHT), image_record.get(JSON_KEY.HEIGHT),
                             "Image has camera height")
            filename = image_record.get(JSON_KEY.FILENAME)
            self.assertIsNotNone(filename, "Filename is defined")
            copied_image_path = path.join(root_path, local_path, filename)
            self.assertTrue(path.isfile(copied_image_path), "Image copied")
            pose_id = image_record.get(JSON_KEY.ID_POSE)
            self.assertIsNotNone(pose_id, "Pose id")
            self.assertTrue(image_record.get(JSON_KEY.USE_POSE_ROTATION_PRIOR), "Use pose rotation prior is true")
            view_center = image_record.get(JSON_KEY.CENTER)
            view_rotation = image_record.get(JSON_KEY.ROTATION)
            self.assertIsNotNone(view_center, "Center of image")
            self.assertIsNotNone(view_rotation, "Rotation of image")
            # poses: extrinsics in openmvg format
            extrinsics = sfm_data.get(JSON_KEY.EXTRINSICS)
            self.assertIsNotNone(extrinsics, "Extrinsics")
            self.assertEqual(4, len(extrinsics), "Trajectory points")
            # Search for the pose of the above image
            pose = None
            for extrinsic in extrinsics:
                if extrinsic.get(JSON_KEY.KEY) == pose_id:
                    pose = extrinsic.get(JSON_KEY.VALUE)
                    break
            self.assertIsNotNone(pose, "Pose for image")
            pose_center = pose.get(JSON_KEY.CENTER)
            pose_rotation = pose.get(JSON_KEY.ROTATION)
            self.assertEqual(view_center, pose_center, "Center are equal")
            self.assertEqual(view_rotation, pose_rotation, "Rotations are equal")
            # Check the openmvg structure
            openmvg_structure = sfm_data.get(JSON_KEY.STRUCTURE)
            kapture_points3d = kapture_sample_data.points3d
            nb_3d_points = kapture_points3d.shape[0]
            self.assertEqual(nb_3d_points, len(openmvg_structure), "All 3D points in openmvg structure")
            # check that all keys are different
            point_idx_set = set()
            for point_3d_structure in openmvg_structure:
                point_idx = point_3d_structure.get(JSON_KEY.KEY)
                self.assertIsNotNone(point_idx, "Point 3D index")
                point_idx_set.add(point_idx)
                point3d_value = point_3d_structure.get(JSON_KEY.VALUE)
                self.assertIsNotNone(point3d_value)
                coords: List[float] = point3d_value.get(JSON_KEY.X)
                self.assertIsNotNone(coords, "3D coordinates")
                observations = point3d_value.get(JSON_KEY.OBSERVATIONS)
                self.assertIsNotNone(observations, "Observations")
            self.assertEqual(nb_3d_points, len(point_idx_set), "All point 3D indexes are different")
            # Check that features and descriptors files are there
            kaptures_images_files = kapture_sample_data.records_camera.data_list()
            for kapture_image_file in kaptures_images_files:
                image_file_name = path.splitext(path.basename(kapture_image_file))[0]
                feature_image_file = path.join(regions_dir_path, image_file_name)+'.feat'
                self.assertTrue(path.isfile(feature_image_file), f"Features file for {image_file_name}")
                descriptor_image_file = path.join(regions_dir_path, image_file_name)+'.desc'
                self.assertTrue(path.isfile(descriptor_image_file), f"Descriptors file for {image_file_name}")

    def test_kapture_export_to_openmvg_reconstruction(self) -> None:
        """
          Test the export_openmvg function on a small kapture dataset with 3D reconstruction data
          by doing a round trip kapture -> openmvg -> kapture conversion
        """

        self.assertTrue(path.exists(self._kapture_sample_path), "Kapture directory exists")
        openmvg_json_file = path.join(self._tempdir.name, 'sfm_export_3d.json')
        openmvg_image_root_path = path.join(self._tempdir.name, 'images')
        regions_dir_path = path.join(self._tempdir.name, 'regions')
        matches_file_path = path.join(self._tempdir.name, 'matches', 'matches.f.txt')
        export_openmvg(self._kapture_sample_path,
                       openmvg_json_file,
                       openmvg_image_root_path,
                       regions_dir_path,
                       matches_file_path,
                       TransferAction.copy)
        self.assertTrue(path.isfile(openmvg_json_file), "Openmvg JSON file created")
        self.assertTrue(path.isdir(regions_dir_path), "Openmvg regions dir created")
        self.assertTrue(path.isfile(matches_file_path), "Openmvg matches file")
        # Reimport the exported data
        import_openmvg(openmvg_json_file, regions_dir_path, matches_file_path, self._kapture_path, TransferAction.copy)
        kapture_data = kcsv.kapture_from_dir(self._kapture_path)
        # Compare to the original sample
        kapture_sample_data = kcsv.kapture_from_dir(self._kapture_sample_path)
        self.assertTrue(equal_kapture(kapture_sample_data, kapture_data), "Kaptures are equal")

    def tearDown(self) -> None:
        """
        Clean up after every test
        """
        self._tempdir.cleanup()


if __name__ == '__main__':
    unittest.main()
